@@ -5,7 +5,7 @@
 
     Highlight code blocks using Pygments.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -35,12 +35,12 @@ from pygments.util import ClassNotFound
 from sphinx.pygments_styles import SphinxStyle, NoneStyle
 
 lexers = dict(
-    none = TextLexer(),
-    python = PythonLexer(),
-    pycon = PythonConsoleLexer(),
-    pycon3 = PythonConsoleLexer(python3=True),
-    rest = RstLexer(),
-    c = CLexer(),
+    none = TextLexer(stripnl=False),
+    python = PythonLexer(stripnl=False),
+    pycon = PythonConsoleLexer(stripnl=False),
+    pycon3 = PythonConsoleLexer(python3=True, stripnl=False),
+    rest = RstLexer(stripnl=False),
+    c = CLexer(stripnl=False),
 )
 for _lexer in lexers.values():
     _lexer.add_filter('raiseonerror')
@@ -183,9 +183,13 @@ class PygmentsBridge(object):
         formatter = self.get_formatter(**kwargs)
         try:
             hlsource = highlight(source, lexer, formatter)
-        except ErrorToken:
+        except ErrorToken as exc:
             # this is most probably not the selected language,
             # so let it pass unhighlighted
+            if warn:
+                warn('Could not parse literal_block as "%s". highlighting skipped.' % lang)
+            else:
+                raise exc
             hlsource = highlight(source, lexers['none'], formatter)
         if self.dest == 'html':
             return hlsource
