@@ -5,7 +5,7 @@
 
     Docutils transforms used by Sphinx when reading documents.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -22,6 +22,7 @@ from sphinx.locale import _, init as init_locale
 from sphinx.util import split_index_msg
 from sphinx.util.nodes import (
     traverse_translatable_index, extract_messages, LITERAL_TYPE_NODES, IMAGE_TYPE_NODES,
+    apply_source_workaround,
 )
 from sphinx.util.osutil import ustrftime
 from sphinx.util.i18n import find_catalog
@@ -120,13 +121,13 @@ class AutoNumbering(Transform):
                 if has_child(node, nodes.caption):
                     self.document.note_implicit_target(node)
             elif isinstance(node, nodes.image):
-                if has_child(node.parent, nodes.caption):
+                if node.parent and has_child(node.parent, nodes.caption):
                     self.document.note_implicit_target(node.parent)
             elif isinstance(node, nodes.table):
                 if has_child(node, nodes.title):
                     self.document.note_implicit_target(node)
             elif isinstance(node, nodes.literal_block):
-                if has_child(node.parent, nodes.caption):
+                if node.parent and has_child(node.parent, nodes.caption):
                     self.document.note_implicit_target(node.parent)
 
 
@@ -155,6 +156,7 @@ class CitationReferences(Transform):
             refnode = addnodes.pending_xref(cittext, reftype='citation',
                                             reftarget=cittext, refwarn=True,
                                             ids=citnode["ids"])
+            refnode.source = citnode.source or citnode.parent.source
             refnode.line = citnode.line or citnode.parent.line
             refnode += nodes.Text('[' + cittext + ']')
             citnode.parent.replace(citnode, refnode)
@@ -169,6 +171,18 @@ TRANSLATABLE_NODES = {
 }
 
 
+class ApplySourceWorkaround(Transform):
+    """
+    update source and rawsource attributes
+    """
+    default_priority = 10
+
+    def apply(self):
+        for n in self.document.traverse():
+            if isinstance(n, nodes.TextElement):
+                apply_source_workaround(n)
+
+
 class ExtraTranslatableNodes(Transform):
     """
     make nodes translatable
@@ -225,7 +239,8 @@ class Locale(Transform):
         dirs = [path.join(env.srcdir, directory)
                 for directory in env.config.locale_dirs]
         catalog, has_catalog = init_locale(dirs, env.config.language,
-                                           textdomain)
+                                           textdomain,
+                                           charset=env.config.source_encoding)
         if not has_catalog:
             return
 
@@ -276,8 +291,10 @@ class Locale(Transform):
                     # document nameids mapping with new name.
                     names = section_node.setdefault('names', [])
                     names.append(new_name)
-                    if old_name in names:
-                        names.remove(old_name)
+                    # Original section name (reference target name) should be kept to refer
+                    # from other nodes which is still not translated or uses explicit target
+                    # name like "`text to display <explicit target name_>`_"..
+                    # So, `old_name` is still exist in `names`.
 
                     _id = self.document.nameids.get(old_name, None)
                     explicit = self.document.nametypes.get(old_name, None)
